/*
 * The most stuf here is taken from Dalles Semiconductor
 * Application Note 187 "1-Wire Search Algorithm".
 *
 * Remarks:    20021027 Stephan Linz <linz@li-pro.net>
 *             adapted to AVR-Ctrl board designed by
 *             http://www.mikrocontroller.com
 */
#include <libac.h>

extern uint8_t	_ow_reset(void);
extern void		_ow_bit_write(uint8_t bit);
extern uint8_t	_ow_bit_read(void);

/*
 * global search state
 */
static uint8_t	LastDiscrepancy,
				LastFamilyDiscrepancy,
				LastDeviceFlag;


/*
 * Perform the 1-Wire search algorithm on the 1-Wire bus
 * using the existing search state.
 */
static uint8_t _ow_rom_search(uint8_t *ROM_NO)
{
	uint8_t	id_bit_number;
	uint8_t	last_zero, rom_byte_number, search_result;
	uint8_t	id_bit, cmp_id_bit;
	uint8_t	rom_byte_mask, search_direction;

	// initialize for search
	id_bit_number	= 1;
	last_zero		= 0;
	rom_byte_number	= 0;
	rom_byte_mask	= 1;
	search_result	= 0;

	// if the last call was not the last one
	if (!LastDeviceFlag) {

		if (!_ow_reset()) {

			// there is no device, reset the search state
			LastDiscrepancy =
				LastDeviceFlag =
				LastFamilyDiscrepancy = 0;

			return 0;
		}

		ow_byte_write(0xf0);		// SEARCH ROM command

		// loop to do search
		do {

			// read a bit and its complement
			id_bit		= _ow_bit_read();
			cmp_id_bit	= _ow_bit_read();

			// check for no devices on 1-Wire
			if ((id_bit == 1) && (cmp_id_bit == 1))
				break;
			else {

				// all devices coupled have 0 or 1
				if (id_bit != cmp_id_bit)
					search_direction = id_bit;	// bit write value for search
				else {

					// if this discrepancy if before the Last Discrepancy
					// on a previous next then pick the same as last time
					if (id_bit_number < LastDiscrepancy)
						search_direction = ((ROM_NO[rom_byte_number] & rom_byte_mask) > 0);
					else
						// if equal to last pick 1, if not then pick 0
						search_direction = (id_bit_number == LastDiscrepancy);

					// if 0 was picked then record its position in LastZero
					if (search_direction == 0) {

						last_zero = id_bit_number;

						// check for Last Discrepancy in family
						if (last_zero < 9)
							LastFamilyDiscrepancy = last_zero;
					}
				}

				// set or clear the bit in the ROM byte rom_byte_number
				// with mask rom_byte_mask
				if (search_direction == 1)
					ROM_NO[rom_byte_number] |= rom_byte_mask;
				else
					ROM_NO[rom_byte_number] &= ~rom_byte_mask;

				// serial number search direction write bit
				_ow_bit_write(search_direction);

				// increment the byte counter id_bit_number
				// and shift the mask rom_byte_mask
				id_bit_number++;
				rom_byte_mask <<= 1;

				// if the mask is 0 then go to new SerialNum byte
				// rom_byte_number and reset mask
				if (rom_byte_mask == 0) {
					/******************* ow_crc8(); *******************/
					rom_byte_number++;
					rom_byte_mask = 1;
				}
			}
		} while (rom_byte_number < sizeof(ow_rom_code_st));	// loop until
															// through all
															// ROM bytes 0..7

		// if the search was successful then
		if (!((id_bit_number < 65) ||
					(ow_crc8(ROM_NO, sizeof(ow_rom_code_st)) != 0))) {

			// search successful so set LastDiscrepancy,
			// LastDeviceFlag, and search_result
			LastDiscrepancy = last_zero;

			// check for last device
			if (LastDiscrepancy == 0)
				LastDeviceFlag = 1;

			search_result = 1;
		}
	}

	// if no device found then reset counters so next 'search' will be
	// like a first
	if (!search_result || !ROM_NO[0]) {
		LastDiscrepancy =
			LastDeviceFlag =
			LastFamilyDiscrepancy =
			search_result = 0;
	}

	return search_result;
}

/*
 * Find the 'first' device on the 1-Wire bus.
 */
static uint8_t _ow_rom_search_first(uint8_t *ROM_NO)
{
	// reset the search state
	LastDiscrepancy =
		LastDeviceFlag =
		LastFamilyDiscrepancy = 0;

	return _ow_rom_search(ROM_NO);
}

/*
 * Find the 'next' device on the 1-Wire bus.
 */
static uint8_t _ow_rom_search_next(uint8_t *ROM_NO)
{
	// leave the search state alone
	return _ow_rom_search(ROM_NO);
}


uint8_t ow_rom_search(void)
{
#if OW_DEVICES > 1
	uint8_t rslt;
#endif
	
	ow_devices_active = 0;

#if OW_DEVICES == 1
	if (!_ow_reset())			// it seems, there is no device anymore
		return 0;

	ow_byte_write(0x33);		// READ ROM command
	if (ow_buf_read((uint8_t *)(&(ow_devices[ow_devices_active].rom_code)),
				sizeof(ow_rom_code_st), 1))
		ow_devices_active	= 1;
#else
	rslt = _ow_rom_search_first(
			(uint8_t *)(&(ow_devices[ow_devices_active].rom_code)));

	while (rslt && (++ow_devices_active < OW_DEVICES)) {
		rslt = _ow_rom_search_next(
				(uint8_t *)(&(ow_devices[ow_devices_active].rom_code)));
	}
#endif

	return ow_devices_active;
}

/*************************************EndOfFile***************************
$Id: $
vim:tw=78:ts=4:sw=4:ai:
*/
